import argparse
import sys
from importlib.metadata import PackageNotFoundError, version
from pathlib import Path

from .anonymize import anonymize_package
from .get_packages import PyprojectTomlNotFoundError, get_packages
from .logger import logger
from .prepare_outdir import prepare_outdir


def get_version(name: str) -> str:
    try:
        return version(name)
    except PackageNotFoundError:
        return 'unknown'


def parse_args():
    parser = argparse.ArgumentParser(
        prog='unname',
        description='Anonymize Python codebases for blind review.',
    )
    parser.add_argument(
        '-o', '--output', type=Path, required=True, help='output directory'
    )
    parser.add_argument(
        '--exclude',
        type=str,
        nargs='*',
        default=['LICEN[CS]E', 'LICEN[CS]E.*'],
        help='glob patterns to exclude from output (default: %(default)s)',
    )
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version=f'{parser.prog} {get_version(parser.prog)}',
    )
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    prepare_outdir(args.output, args.exclude)

    try:
        packages = get_packages(args.output)
    except PyprojectTomlNotFoundError as e:
        logger.error(e)
        return sys.exit(1)

    for package in packages:
        anonymize_package(package)


if __name__ == '__main__':
    main()



from pathlib import Path

import tomlkit
import tomlkit.api  # necessary to make type checkers happy

from .logger import logger


def anonymize_package(path: Path) -> None:
    anonymize_pyproject_toml(path / 'pyproject.toml')
    anonymize_readme_md(path / 'README.md')


def anonymize_pyproject_toml(path: Path) -> None:
    try:
        with open(path, 'r') as f:
            content = f.read()
    except FileNotFoundError:
        logger.warning(f'pyproject.toml not found at {path}, skipping')
        return

    anonymized_content = anonymize_pyproject_toml_content(content)

    with open(path, 'w') as f:
        f.write(anonymized_content)


def anonymize_pyproject_toml_content(content: str) -> str:
    doc = tomlkit.parse(content)

    project = doc['project']
    if not isinstance(project, tomlkit.api.Table):
        raise ValueError('invalid pyproject.toml: project is not a table')

    if 'authors' in project:
        project.remove('authors')
    if 'maintainers' in project:
        project.remove('maintainers')
    if 'urls' in project:
        project.remove('urls')

    anonymized_content = tomlkit.dumps(doc)
    return anonymized_content


class ReadmeParseError(Exception):
    pass


def anonymize_readme_md(path: Path) -> None:
    try:
        with open(path, 'r') as f:
            content = f.read()
    except FileNotFoundError:
        logger.warning(f'README.md not found at {path}, skipping')
        return

    anonymized_content = anonymize_readme_md_content(content)

    with open(path, 'w') as f:
        f.write(anonymized_content)


def anonymize_readme_md_content(content: str) -> str:
    def is_begin(line: str) -> bool:
        return line.strip().lower() == '<!-- begin-unname -->'

    def is_end(line: str) -> bool:
        return line.strip().lower() == '<!-- end-unname -->'

    lines = content.splitlines(keepends=True)
    anonymized_lines = []
    count = 0
    for line in lines:
        if is_begin(line):
            count += 1

        match count:
            case 0:
                anonymized_lines.append(line)
            case 1:
                pass
            case _:
                raise ReadmeParseError(
                    'Unmatched begin-unname/end-unname markers'
                )

        if is_end(line):
            count -= 1

    anonymized_content = ''.join(anonymized_lines)
    return anonymized_content



from pathlib import Path
from typing import cast

import tomlkit


class PyprojectTomlNotFoundError(Exception):
    pass


def get_packages(outdir: Path) -> list[Path]:
    try:
        with open(outdir / 'pyproject.toml', 'r') as f:
            pyproject = tomlkit.load(f)
    except FileNotFoundError:
        raise PyprojectTomlNotFoundError('Root pyproject.toml not found')

    package_globs = (
        cast(
            list[str] | None,
            pyproject.get('tool', {})
            .get('uv', {})
            .get('workspace', {})
            .get('members'),
        )
        or []
    )
    packages = [outdir] + [
        package
        for package_glob in package_globs
        for package in outdir.glob(package_glob)
    ]
    assert all(package.exists() for package in packages)
    return packages
